program W2V

!---------------------------------------------------------------------------------------------
  ! Reorders the Hamiltonian matrix generated by Wannier90 in the format
  ! approved by the program NanoTCADViDES.
  ! 
  ! Input:
  !
  ! It requires as input the following variables:
  ! arg      : equal to seedname defined in the previous Wannier90 calculation.
  ! cutoff   : the largest distance between two WFs for which the Hamiltonian matrix element is retained and used. Units are A.
  ! length   : length of the NR in A
  !
  ! The program READS the Hamiltonian matrix and the coordinates of final WFs center in the 
  ! Wannier90 output files 'seedname'_hr.dat and 'seedname'_centres.xyz, respectively,
  ! and READS the lattice vector in the corresponding Wannier90 input file 'seedname'.win
  !
  ! Output:
  !
  ! The Hamiltonian matrix in the format valid for NanoTCADViDES is written to stardard output Wannier2ViDES.H. 
  ! The atomic coordinates of all atoms of whole NR are written to standard output Wannier2ViDES.xyz 
!---------------------------------------------------------------------------------------------

implicit none

integer, parameter :: num_ao=1            !number of atomic orbital for each atom
integer, parameter :: num_wann_max=30     !maximum number of WFs 

integer :: i,j,k,kk,l,n,m,num_wann,num_k
integer :: imax,jmax
real(8), dimension(3,3) :: alat
real(8) :: d,diff
real(8) :: cutoff     !The largest distance between two WFs for which the Hamiltonian matrix element is retained and used. Units are A.
real(8) :: length     !length of the NR in A
real(8), dimension(num_wann_max) :: x,y,z
real(8), dimension(num_wann_max) :: xa,ya,za
real(8), dimension(600,600) :: htb
real(8), dimension(-200:200,num_wann_max,num_wann_max) :: hr,hi
character(1), dimension(num_wann_max) :: atom
character(20) :: row
character(32) :: arg

!CALL GETARG(1,arg)

write(*,'(5x,a,$)') 'seedname >>'
read(*,*) arg 
write(*,'(5x,a,$)') 'length of NR in A >>'
read(*,*) length
write(*,'(5x,a,$)') 'Dist cutoff value in A >>'
read(*,*) cutoff

open(unit=7,file=trim(adjustl(arg))//'_hr.dat',form='formatted',status='old')
open(unit=8,file=trim(adjustl(arg))//'_centres.xyz',form='formatted',status='old')
open(unit=9,file=trim(adjustl(arg))//'.win',form='formatted',status='old')

open(unit=10,file='Wannier2ViDES.H',form='formatted',status='unknown')
open(unit=11,file='Wannier2ViDES.xyz',form='formatted',status='unknown')
open(unit=12,file='n_Nc.dat',form='formatted',status='unknown')

write(10,*) num_ao,'    0    0.000000    0.000000'

read(7,*) 
read(7,*) num_wann !number of WFs
read(7,*) num_k    !number of Wigner-Seitz grid-points nrpts.

if(num_wann > num_wann_max) write(6,*) 'WARNING: number of WFs exceeded. Maximum value allowed is num_wann_max defined at line 4'

if(nint(num_k/15.d0) == num_k/15.d0) n = nint(num_k/15.d0)
if(nint(num_k/15.d0) .ne. num_k/15.d0) n = nint(num_k/15.d0)+1
do i = 1,n
   read(7,*) 
enddo

! ###################################################################
!Reads the coordinates x,y,z of final WFs centres and calculates the mutual distances
!WARNING: The NR is in the plane yz. z is the transport direction.
!         The unit cell has symmetry Orthorhombic P 

do i = 1,100000
   read(9,13) row 
   if(row == 'Begin Unit_Cell_Cart') exit
enddo

read(9,*) alat(1,1),alat(1,2),alat(1,3)  !in A    real lattice vector v(1) 
read(9,*) alat(2,1),alat(2,2),alat(2,3)  !in A    real lattice vector v(2)
read(9,*) alat(3,1),alat(3,2),alat(3,3)  !in A    real lattice vector v(3)

read(8,*)
read(8,*)
do i = 1,num_wann
   read(8,8) x(i),y(i),z(i)
enddo

do i = 1,num_wann
   read(8,9) atom(i),xa(i),ya(i),za(i)
   if(abs(y(i)-ya(i)) > 1.d0) write(6,*) i,x(i),y(i),z(i)
   if(abs(y(i)-ya(i)) > 1.d0 .and. y(i) < ya(i)) then
        y(i) = y(i) + alat(2,2)
        write(6,*) y(i)
   endif
   if(abs(y(i)-ya(i)) > 1.d0 .and. y(i) > ya(i)) then
        y(i) = y(i) - alat(2,2)
        write(6,*) y(i)
   endif
enddo


! ###################################################################

! ###################################################################
!Reads the Hamiltonian matrix and retains the elements referred to two WFs closer than cutoff
hr(:,:,:) = 0.d0
n = (num_k-1)/2

do k = -n,n
   do j = 1,num_wann
      do i = 1,num_wann
         read(7,10) hr(k,i,j),hi(k,i,j)
         d = dsqrt((x(i)-x(j))**2 + (y(i)-y(j))**2 + (z(i)-z(j)-k*alat(3,3))**2)
         if(abs(d) - cutoff > 1D-8) then
           hr(k,i,j) = 0.d0 
           hi(k,i,j) = 0.d0 
         endif
      enddo
   enddo
enddo
! ###################################################################

! ###################################################################
num_k = int(length / alat(3,3))
if(mod(num_k,4) == 0.d0) then
  go to 30
else
 diff = dfloat(num_k)/4 - int(num_k/4)    
 if(diff .ge. 0.5d0) num_k = num_k + (1-diff)*4  
 if(diff .lt. 0.5d0) num_k = num_k - (1-diff)*4  
 write(6,*) 'num_k',num_k
endif
30 n = num_k/2
write(6,*) 'num_k, number of Unit cell considered in the NR in the z direction', num_k
write(6,*) 'N,Nc',num_ao*num_wann,num_k
write(12,*) num_ao*num_wann,num_k

! Writes out the diagonal of the Hamiltonian matrix
do l = 0,num_k-1 
   do i = 1,num_wann
      k = 0
      do m = l,num_k-1
         do j = 1,num_wann
            if(j+m*num_wann == i+l*num_wann) then
               if(k > n) then
                  kk = k - 2*n       !num_k e' pari
                  if(abs(hr(kk,i,j)) > 1e-6) write(10,11) i+l*num_wann,j+m*num_wann,hr(kk,i,j),hi(kk,i,j)
               else
                  if(abs(hr(k,i,j)) > 1e-6) write(10,11) i+l*num_wann,j+m*num_wann,hr(k,i,j),hi(k,i,j)
               endif
            endif
         enddo !j
         k = k+1
      enddo !m
   enddo !i
enddo !l

! Writes out the upper triangle of the Hamiltonian matrix
do l = 0,num_k-1 
   do i = 1,num_wann
      k = 0
      do m = l,num_k-1
         do j = 1,num_wann
            if(j+m*num_wann > i+l*num_wann) then
               if(k < n) then
                  kk = k - 2*n       !num_k e' pari
                  if(abs(hr(k,i,j)) > 1e-6) write(10,11) i+l*num_wann,j+m*num_wann,hr(k,i,j),hi(k,i,j)
               endif
            endif
         enddo !j
         k = k+1
      enddo !m
   enddo !i
enddo !l

do j = 0,num_k-1
do i = 1,num_wann
write(11,12) atom(i),xa(i),ya(i),za(i)+j*alat(3,3)
enddo
enddo

8  format (9X,F12.8,5X,F12.8,5X,F12.8)
9  format (1A,7X,3(F13.8,4X))
10 format (28X,F9.6,3X,F9.6)
11 format (2X,I5,2X,I5,2X,F9.6,2X,F9.6)
12 format (A1,2X,3(F12.7,2X))
13 format(A20)

stop
end

